# -*- coding: utf-8 -*-
#
# Copyright (c) 2021, HILSTER GmbH, Germany - https://hilster.io
# All rights reserved.
#

import sys
import argparse
import webbrowser
import os.path
import hlm.licenses
import shutil
import hlm
import argcomplete

if sys.argv[0].endswith("__main__.py"):
    sys.argv[0] = "python -m hlm"

if sys.argv[0].endswith("hlm-script.py"):
    sys.argv[0] = "hlm"


class HLMCli:
    def __init__(self):
        self.command = None
        self.parse_args()
        handler = self.get_handler()
        status_code = handler()

        if status_code is None:
            status_code = 0

        sys.exit(status_code)

    def parse_args(self):
        parser = argparse.ArgumentParser(
            description='hlm - HILSTER License Manager (https://hilster.io)'
        )

        subparsers = parser.add_subparsers(dest="command", help='the command')
        subparsers.required = True

        hostid_parser = subparsers.add_parser('hostid', help='query host id')
        list_parser = subparsers.add_parser('list', help='list licenses available on this host')

        purge_parser = subparsers.add_parser('purge', help='purge all licenses and credentials')
        purge_parser.add_argument("-f", "--force", action="store_true", default=False,
                                  help="Does not ask the user")

        config_parser = subparsers.add_parser('config',
                                              help='set user information sent to the license server')

        dongles_parser = subparsers.add_parser('dongles', help='list dongles')

        # activation
        activate_parser = subparsers.add_parser('activate',
                                                help="activate this node using the supplied activation keys")
        activate_parser.add_argument("-s", "--server",
                                     default="https://licenses.hilster.io",
                                     help="license server url")
        activate_parser.add_argument("-o", "--output",
                                     help="output directory for generated license file")
        activate_parser.add_argument("-f", "--filename",
                                     help="license filename for generated license file")
        activate_parser.add_argument("--insecure",
                                     action="store_true",
                                     help="disable ssl certificate check (can be useful behind some proxy servers)")
        activate_parser.add_argument("activationkeys", nargs="+", help="activation keys")

        # renewal
        renew_parser = subparsers.add_parser('renew', help="renew all activated licenses")
        renew_parser.add_argument("-s", "--server",
                                  default="https://licenses.hilster.io",
                                  help="license server url")
        renew_parser.add_argument("--insecure",
                                  action="store_true",
                                  help="disable ssl certificate check (can be useful behind some proxy servers)")

        # unlock
        unlock_parser = subparsers.add_parser('unlock', help="unlock a product for testing purposes")
        unlock_parser.add_argument("product", help="the product to be unlocked")
        unlock_parser.add_argument("version", help="the product version to be unlocked")

        docs_parser = subparsers.add_parser('docs', help='open documentation')
        version_parser = subparsers.add_parser('version', help='show version')

        import_parser = subparsers.add_parser('import', help='import a license file')
        import_parser.add_argument('filename', help='license file')
        import_parser.add_argument('-f', '--force', action="store_true", default=False,
                                   help='overwrite existing license files')

        license_parser = subparsers.add_parser('licenses', help="show open-source licenses")

        complete_parser = subparsers.add_parser('complete', help="print an evaluable expression for bash completion "
                                                                 "(use with 'eval $(htf complete)')")

        argcomplete.autocomplete(parser)

        parser.parse_args(namespace=self)

    def get_handler(self):
        handler = getattr(self, "handle_{}".format(self.command), None)
        if handler is None:
            raise Exception("Unknown command {!r}".format(self.command))
        return handler

    @staticmethod
    def handle_complete():
        print("eval \"$(register-python-argcomplete hlm)\"")

    @staticmethod
    def handle_hostid():
        hlm.core.print_hostid()

    @staticmethod
    def handle_list():
        hlm.core.print_available_licenses()

    def handle_purge(self):
        hlm.core.purge(self.force)

    @staticmethod
    def handle_config():
        hlm.core.set_user_information()

    @staticmethod
    def handle_dongles():
        print("Currently attached usb dongles:\n")
        for serial, dongleid in hlm.core.get_dongles_iterator():
            print(serial + ":", dongleid)
        print("")

    def handle_activate(self):
        if "HLM_INSECURE" in os.environ:
            self.insecure = True

        return hlm.activate_keys(
            self.server, self.activationkeys,
            output_directory=self.output,
            filename=self.filename,
            insecure=self.insecure,
            raise_exceptions=False
        )

    def handle_renew(self):
        if "HLM_INSECURE" in os.environ:
            self.insecure = True

        return hlm.renew(self.server, self.insecure)

    def handle_unlock(self):
        hlm.core.unlock(self.product, self.version)

    @staticmethod
    def handle_docs():
        url = "https://docs.hilster.io/hlm/hlm-{}/".format(hlm.__version__)
        webbrowser.open(url)

    def handle_import(self):
        filename = os.path.basename(self.filename)
        print("Importing license", self.filename)
        destination_folder = os.path.dirname(hlm.licenses.__file__)
        destination = os.path.join(destination_folder, filename)
        if os.path.exists(destination):
            if not self.force:
                raise Exception("License file already exists: %s. Use '-f' option to overwrite." % destination)

        print("Copying %s to %s" % (self.filename, destination))
        shutil.copy(self.filename, destination)

    @staticmethod
    def handle_licenses():
        print(LICENSES)

    @staticmethod
    def handle_version():
        print("hlm-" + hlm.__version__)


LICENSES = """
Open Source Licenses
====================

The Sodium Crypto library (libsodium-1.0.18, http://libsodium.org)
-----------------------------------------------------------

Copyright (c) 2013-2019
Frank Denis <j at pureftpd dot org>

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
"""


def main():
    HLMCli()


if __name__ == "__main__":
    main()
